;; These 2 functions are used only during Debugging!
;; MsgDD is somewhat similar to the dw2str function, it basically just converts the input falue to a string representation!
;; Then it copies the value to the clipboard, I did this cause I usually use this value in a calculation or to paste in Notepad
;; Then show the value in a messagebox. The function also preserves the registers and flags, so it can be used anywhere in code!
;; I use it extensivesly when debugging the check values as they go by.
;; eg. invoke MsgDD, eax
;; This will display the value in eax for me and won't affect the program so I can use it anywhere!
MsgDD PROC Val  :DWORD ;USES eax ecx edx esi edi Val :DWORD
  LOCAL hAlloc  :DWORD 

  pushad ; Push all registers
  pushfd ; Push all flags

  ;; NB Memory used for the clipboard is allocated with GlobalAlloc and NOT freed!
  .if mCall(GlobalAlloc, GMEM_MOVEABLE and GMEM_DDESHARE, 11) == NULL
    mGetLastError "Error allocating memory for MsgDD", "GlobalAlloc Error"
  .elseif
    mov hAlloc, eax

    mov   edi, hAlloc ; edi stores the beginning offset of our string

    mov   eax, Val      ; eax now contains input value for division later
    mov   esi, edi      ; copy the offset of our string to esi (now both esi & ecx contain the offset)
    mov   ecx, 0Ah      ; 0Ah == 10 == Decimal system (similar to Radix parameter of _itoa)

  @@:
    xor   edx, edx            ; Clear edx for division
    div   ecx                 ; Divide eax by 10 (Radix)
    add   dl, 30h             ; Add 48 to the value (This is value for 0 (zero) on ASCII table)
    mov   byte ptr [edi], dl  ; Move our new ASCII value into string position
    inc   edi                 ; Move to next string value
    test  eax, eax            ; Test to see if we've reached 0 (zero)
    jnz   short @B            ; If (!Zero?) Then ... Go Back to beginning
    mov   byte ptr [edi], 0   ; Place our terminating zero into position
    dec   edi                 ; Decrement ecx so we can begin reversing string

  @@:                   ; This section reverses our string
    mov   dl, [esi]
    mov   al, [edi]
    mov   [edi], dl
    dec   edi
    mov   [esi], al
    inc   esi
    cmp   esi, edi
    jb    short @B

    ;;Clipboard
    .if mCall(OpenClipboard, NULL) == NULL
      mGetLastError "", "OpenClipboard Error"
    .else
      .if mCall(EmptyClipboard) == NULL
        mGetLastError "", "EmptyClipboard Error"
      .else
        .if mCall(SetClipboardData, CF_TEXT, hAlloc) == NULL
          mGetLastError "", "SetClipboardData Error"
        .endif
      .endif
      .if mCall(CloseClipboard) == NULL
        mGetLastError "", "CloseClipboard Error"
      .endif
    .endif

    invoke MessageBox, NULL, hAlloc, sz("MsgDD"), MB_OK  ; Finally display our message
  .endif
  popfd
  popad
  ret
MsgDD ENDP

MsgHex PROC Val :DWORD
  LOCAL hAlloc  :DWORD 

  pushad ; Push all registers
  pushfd ; Push all flags

  ;; NB Memory used for the clipboard is allocated with GlobalAlloc and NOT freed!
  .if mCall(GlobalAlloc, GMEM_MOVEABLE and GMEM_DDESHARE, 11) != NULL
    mov hAlloc, eax

    mov   edi, hAlloc   ; edi stores the beginning offset of our string

    mov   eax, Val      ; eax now contains input value for division later
    mov   esi, edi      ; copy the offset of our string to esi (now both esi & ecx contain the offset)
    mov   ecx, 10h      ; 10h == 16 == Hex system (similar to Radix parameter of _itoa)

  @@:
    xor   edx, edx            ; Clear edx for division
    div   ecx                 ; Divide eax by 16 (Radix)
    .IF (dl > 9)
      sub   dl, 9
      add   dl, 40h
    .ELSE
      add   dl, 30h
    .ENDIF
    mov   byte ptr [edi], dl  ; Move our new ASCII value into string position
    inc   edi                 ; Move to next string value
    test  eax, eax            ; Test to see if we've reached 0 (zero)
    jnz   short @B            ; If (!Zero?) Then ... Go Back to beginning
    mov   byte ptr [edi], 0   ; Place our terminating zero into position
    dec   edi                 ; Decrement ecx so we can begin reversing string

  @@:                   ; This section reverses our string
    mov   dl, [esi]
    mov   al, [edi]
    mov   [edi], dl
    dec   edi
    mov   [esi], al
    inc   esi
    cmp   esi, edi
    jb    short @B

    ;;Clipboard
    .if mCall(OpenClipboard, NULL) == NULL
      mGetLastError "", "OpenClipboard Error"
    .else
      .if mCall(EmptyClipboard) == NULL
        mGetLastError "", "EmptyClipboard Error"
      .else
        .if mCall(SetClipboardData, CF_TEXT, hAlloc) == NULL
          mGetLastError "", "SetClipboardData Error"
        .endif
      .endif
      .if mCall(CloseClipboard) == NULL
        mGetLastError "", "CloseClipboard Error"
      .endif
    .endif

    invoke MessageBox, NULL, hAlloc, sz("MsgHex"), MB_OK  ; Finally display our message
  .elseif
    mGetLastError "Error allocating memory for MsgHex", "GlobalAlloc Error"
  .endif
  popfd
  popad
  ret
MsgHex ENDP

;; This function is an extention to the above function.
;; I wrote it cause sometimes I want to display a message with my value,
;; cause perhaps I'm displaying several values and if I don't display a small message with it, I won't know where I am!
;; eg. invoke MsgTxtDD, eax, sz("WinSock generated the following Error: "), sz("WinSock Error")
;; Very useful to me when debugging! So the above will show a message and the value returned
;; Also preserves registers and flags! Only really useful during debugging stage!
MsgTxtDD PROC inVal :DWORD, pText:DWORD, pTitle :DWORD
  LOCAL pAlloc :DWORD 
  LOCAL pFinalAlloc :DWORD 

  pushad ; Push all registers
  pushfd ; Push all flags

  ;; NB Memory used for the clipboard is allocated with GlobalAlloc and NOT freed!
  .if mCall(GlobalAlloc, GMEM_MOVEABLE and GMEM_DDESHARE, 11) == NULL
    mGetLastError "Error allocating memory for MsgTxtDD", "GlobalAlloc Error"
  .elseif
    mov pAlloc, eax

    mov   edi, pAlloc ; edi stores the beginning offset of our string

    mov   eax, inVal      ; eax now contains input value for division later
    mov   esi, edi      ; copy the offset of our string to esi (now both esi & ecx contain the offset)
    mov   ecx, 0Ah      ; 0Ah == 10 == Decimal system (similar to Radix parameter of _itoa)

  @@:
    xor   edx, edx            ; Clear edx for division
    div   ecx                 ; Divide eax by 10 (Radix)
    add   dl, 30h             ; Add 48 to the value (This is value for 0 (zero) on ASCII table)
    mov   byte ptr [edi], dl  ; Move our new ASCII value into string position
    inc   edi                 ; Move to next string value
    test  eax, eax            ; Test to see if we've reached 0 (zero)
    jnz   short @B            ; If (!Zero?) Then ... Go Back to beginning
    mov   byte ptr [edi], 0   ; Place our terminating zero into position
    dec   edi                 ; Decrement ecx so we can begin reversing string

  @@:                   ; This section reverses our string
    mov   dl, [esi]
    mov   al, [edi]
    mov   [edi], dl
    dec   edi
    mov   [esi], al
    inc   esi
    cmp   esi, edi
    jb    short @B

    ;;Clipboard
    .if mCall(OpenClipboard, NULL) == NULL
      mGetLastError "", "OpenClipboard Error"
    .else
      .if mCall(EmptyClipboard) == NULL
        mGetLastError "", "EmptyClipboard Error"
      .else
        .if mCall(SetClipboardData, CF_TEXT, pAlloc) == NULL
          mGetLastError "", "SetClipboardData Error"
        .endif
      .endif
      .if mCall(CloseClipboard) == NULL
;        mGetLastError "", "CloseClipboard Error"
      .endif
    .endif

    .if pText == NULL
      .if pTitle == NULL
        IFNDEF szMsgTxtDDTitle
          .const
            szMsgTxtDDTitle db "MsgTxtDD", 0
          .code
        ENDIF
        invoke MessageBox, NULL, pAlloc, addr szMsgTxtDDTitle, MB_OK  ; Finally display our message
      .else
        invoke MessageBox, NULL, pAlloc, pTitle, MB_OK  ; Finally display our message
      .endif
    .else
      add mCall(lstrlen, pText), 11
      mov pFinalAlloc, mAlloc(eax)
      invoke lstrcat, pFinalAlloc, pText
      invoke lstrcat, pFinalAlloc, pAlloc

      .if pTitle == NULL
        IFNDEF szMsgTxtDDTitle
          .const
            szMsgTxtDDTitle db "MsgTxtDD", 0
          .code
        ENDIF
        invoke MessageBox, NULL, pFinalAlloc, addr szMsgTxtDDTitle, MB_OK  ; Finally display our message
      .else
        invoke MessageBox, NULL, pFinalAlloc, pTitle, MB_OK  ; Finally display our message
      .endif
      mFree(pFinalAlloc)
    .endif
  .endif
  popfd
  popad
  ret
MsgTxtDD ENDP
